import rpyc

from .raw_memory import BoardRawMemory


class MyService(rpyc.Service):
    def on_connect(self):
        print("CONNECTION")
        self._mem = BoardRawMemory()
        self._conn._config.update(
            dict(
                allow_all_attrs=True,
                allow_pickle=False,
                allow_getattr=True,
                allow_setattr=True,
                allow_delattr=True,
                import_custom_exceptions=True,
                instantiate_custom_exceptions=True,
                instantiate_oldstyle_exceptions=True,
            )
        )

    def exposed_mem(self):
        return self._mem


if __name__ == "__main__":
    from rpyc.utils.server import ThreadedServer

    print("START SERVER")
    t = ThreadedServer(MyService, port=18861)
    t.start()



class MemoryInterface(object):
    def __init__(self, addr_base=0x00000000, parent_memory=None):
        self.addr_base = addr_base
        self._parent = parent_memory

    def read(self, addr):
        """Read one register"""
        return self._parent.read(self.addr_base + addr)

    def reads(self, addr, length):
        return self._parent.reads(self.addr_base + addr, length)

    def write(self, addr, value):
        """Write one register"""
        self._parent.write(self.addr_base + addr, value)

    def writes(self, addr, value):
        self._parent.writes(self.addr_base + addr, value)

    def setbit(self, addr, bitnumber):
        v = self.read(addr) | (0x00000001 << bitnumber)
        self.write(addr, v)
        return v

    def clrbit(self, addr, bitnumber):
        v = self.read(addr) & (~(0x00000001 << bitnumber))
        self.write(addr, v)
        return v

    def changebit(self, addr, bitnumber, v):
        if v:
            return self.setbit(addr, bitnumber)
        else:
            return self.clrbit(addr, bitnumber)

    def bitstate(self, addr, bitnumber):
        return bool(self.read(addr) & (0x00000001 << bitnumber))

    def to_pyint(self, v, bitlength=14):
        v = v & (2**bitlength - 1)
        if v >> (bitlength - 1):
            v = v - 2**bitlength
        return int(v)

    def from_pyint(self, v, bitlength=14):
        if v < 0:
            v = v + 2**bitlength
        v = v & (2**bitlength - 1)
        return int(v)



from distutils.core import Extension, setup
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.rst").read_text()

setup(
    name="pyrp3",
    version="1.1.0",
    description="Python utilities for redpitaya",
    author="Pierre Cladé",
    author_email="pierre.clade@upmc.fr",
    maintainer="Bastian Leykauf",
    maintainer_email="leykauf@physik.hu-berlin.de",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    packages=["pyrp3", "pyrp3.enum"],
    install_requires=["myhdl", "rpyc", "cached_property", "numpy"],
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Other Audience",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords=["redpitaya", "FPGA", "zynq"],
    ext_modules=[Extension("monitor", ["monitor/monitor.c"])],
)
